<?php
require_once(PATH_tslib.'class.tslib_pibase.php');
require_once(t3lib_extMgm::extPath('mageconnect') . 'lib/class.tx_mageconnect_helper.php');
require_once(t3lib_extMgm::extPath('mageconnect') . 'lib/class.tx_mageconnect_soap.php');
require_once(t3lib_extMgm::extPath('mageconnect') . 'lib/class.tx_mageconnect_cache.php');

class tx_mageconnect_flexform extends tslib_pibase
{
	const cacheId = 'productsList';
	
	protected $_products = array();
	protected $_soap;
	protected $_helper;
	protected $_cache;
	
	function __construct()
	{
		$this->_helper 	= t3lib_div::makeInstance('tx_mageconnect_helper');
	}
	
	function _connect()
	{
		$soapUrl 	= $this->_helper->getModuleConfig('magento_url');
		$soapUser 	= $this->_helper->getModuleConfig('magento_api_username');
		$soapKey 	= $this->_helper->getModuleConfig('magento_api_key');
		
		$this->_soap = t3lib_div::makeInstance('tx_mageconnect_soap', 
											   $soapUrl, 
											   $soapUser, 
											   $soapKey
		);
	}
	
	/**
	 * initCache
	 * initialize Caching Instance
	 */
	function initCache()
	{
		$this->_cache = t3lib_div::makeInstance('tx_mageconnect_cache');
	}

	/**
	 * _getFlexformConfig
	 * Get the flexform config by typename and fieldname
	 * 
	 * @param string $type
	 * @param string $field
	 * @return string
	 */
	function _getFlexformConfig($field, $type)
	{
		$flexform = t3lib_div::xml2array($this->conf['row']['pi_flexform']);
		$languageKey = 'lDEF';
		
		if (array_key_exists('data', $flexform) && !empty($flexform['data']) && $flexform['data']) {
			return $flexform['data'][$type][$languageKey][$field]['vDEF'];
		}
		return '';
	}


	/**
	 * populateProductList
	 * Populates the product option list in flexform
	 * 
	 * @param array $config
	 * @param array $transferdata
	 * @return array
	 */
	function populateProductList($config, $transferdata)
	{
		$this->conf = $config;
		
		$this->initCache();
		
		// Cache Lifetime
		$lifetime = $this->_helper->getModuleConfig('cache_lifetime');
		
		// Store View Code from Flexform
		$store_view_code = $this->_getFlexformConfig('store_view_code', 'product_setting');
		
		// Cache ID for Store View Code
		$cacheId = self::cacheId . '_' . $store_view_code;
		
		if (FALSE === ($optionList = $this->_cache->get($cacheId))) {
			
			$optionList = array();
		
			if (empty($this->_products)) {
					
				try {
					$this->_connect();
					$this->_products = $this->_soap->getProductsList($store_view_code);
				} catch (Exception $e) {
					die('Could not fetch products list: ' . $e);
				}
				
			} 
			
			foreach ($this->_products as $_product) {
					
				$productId 		= $_product->product_id;
				$productSku		= $_product->sku;
				$productName 	= $_product->name;
				
				$optionStr = '['.$productId.'] ' . $productName;
				
				$optionList[] = array(0 => $optionStr, 1 => $productId);
			}
			$this->_cache->set( $cacheId, $optionList, array(), $lifetime );
		
		} 
		$config['items'] = array_merge($config['items'],$optionList);
		return $config;	
	}
}

